@@ -21,7 +21,7 @@
 
 #include <stdlib.h>
 
-#define myRandf(x) (((float)(x) * random()) / 2147483647)
-#define myRandi(x) (int)(myRandf(x))
+#define rsRandf(x) (((float)(x) * random()) / 2147483647)
+#define rsRandi(x) (int)(rsRandf(x))
 
 #endif
