@@ -62,6 +62,7 @@ int text_y = 70;
 
 // Scene position
 #define Z_Off -128.0f
+#define Z_Depth 8
 
 unsigned char speed[MAX_TEXT_X];
 unsigned char text[MAX_TEXT_X * MAX_TEXT_Y + MAX_TEXT_X];
@@ -82,99 +83,125 @@ int pic_fade = 0;
 #ifdef HAVE_IMAGEMAGICK
 #include <magick/api.h>
 #include <dirent.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <unistd.h>
 
 char *dirName = NULL;
 DIR *imageDir;
 
-	#ifdef HAVE_PTHREADS
-		#include <pthread.h>
-		pthread_t *imageLoadingThread = NULL;
-		pthread_mutex_t next_pic_mutex;
-		pthread_cond_t next_pic_cond;
-		unsigned char *next_pic = NULL;
-		int exiting = 0;
-	#endif
+unsigned char *next_pic = NULL;
+
+// Directory scanning + image loading code in a separate function callable either from loadNextImage or another thread if pthreads is available.
+void loadNextImageFromDisk() {
+	ExceptionInfo exception;
+	Image *image = NULL, *scaled_image;
+	ImageInfo *image_info;
+	int dirLoop = 0;
+
+	GetExceptionInfo (&exception);
+
+	do {
+		struct dirent *file;
+
+		if (!imageDir) {
+			if (dirLoop) {
+				dirName = NULL;
+				return;
+			}
+
+			imageDir = opendir (dirName);
+			dirLoop = 1;
+		}
+
+		file = readdir (imageDir);
+		if (file) {
+			struct stat fileStat;
+			char *full_path_and_name = (char *)malloc(strlen(dirName) + 1 + strlen(file->d_name) + 1);
+
+			if (full_path_and_name) {
+				sprintf(full_path_and_name, "%s/%s", dirName, file->d_name);
+
+				if (!stat(full_path_and_name, (struct stat *)&fileStat)) {
+					if (S_ISREG(fileStat.st_mode)) {
+						FILE *imagefh = fopen(full_path_and_name, "r");
+
+						if (imagefh) {
+							image_info = CloneImageInfo ((ImageInfo *) NULL);
+							image_info->file = imagefh;
+							image = ReadImage (image_info, &exception);
+
+							if (exception.severity != UndefinedException) {
+								fprintf (stderr, "Error loading %s: %s\n", full_path_and_name, exception.reason);
+							}
+						} else {
+							fprintf (stderr, "Unable to open %s: %s\n", full_path_and_name, exception.reason);
+						}
+					}
+				}
+
+				free(full_path_and_name);
+			} else {
+				fprintf (stderr, "Out of memory\n");
+			}
+		} else {
+			imageDir = NULL;
+		}
+	} while (!image);
+
+	scaled_image = ScaleImage (image, text_x, text_y, &exception);
+	NormalizeImage(scaled_image);
+	ContrastImage(scaled_image, 0);
+	NegateImage(scaled_image, 0);
+
+	if (!next_pic)
+		next_pic = (unsigned char *)malloc (text_x * text_y);
+
+	ExportImagePixels (scaled_image, 0, 0, text_x, text_y, "I", CharPixel, next_pic, &exception);
+
+	DestroyImage (image);
+	DestroyImage (scaled_image);
+}
+
+#ifdef HAVE_PTHREADS
+#include <pthread.h>
+pthread_t *imageLoadingThread = NULL;
+pthread_mutex_t *next_pic_mutex;
+pthread_cond_t *next_pic_cond;
+int exiting = 0;
+
+void *imageLoadingThreadMain() {
+	do {
+		loadNextImageFromDisk();
+
+		pthread_cond_wait(next_pic_cond, next_pic_mutex);
+	} while (!exiting);
+
+	pthread_exit(NULL);
+}
+#endif
 
 #endif
 
+
 void loadNextImage ()
 {
 #ifdef HAVE_IMAGEMAGICK
 	if (dirName) {
-#ifndef HAVE_PTHREADS
-		unsigned char *next_pic = NULL;
-#endif
-		// Directory scanning + image loading code in a separate function callable either from loadNextImage or another thread if pthreads is available.
-		void loadNextImageFromDisk() {
-			ExceptionInfo exception;
-			Image *image = NULL, *scaled_image;
-			ImageInfo *image_info;
-			int dirLoop = 0;
-
-			GetExceptionInfo (&exception);
-
-			do {
-				struct dirent *file;
-
-				if (!imageDir) {
-					if (dirLoop) {
-						dirName = NULL;
-						return;
-					}
-
-					imageDir = opendir (dirName);
-					dirLoop = 1;
-				}
-
-				file = readdir (imageDir);
-				if (file) {
-					image_info = CloneImageInfo ((ImageInfo *) NULL);
-					sprintf (image_info->filename, "%s/%s", dirName, file->d_name);
-					image = ReadImage (image_info, &exception);
-
-					if (exception.severity != UndefinedException) {
-						fprintf (stderr, "Error loading %s/%s: %s\n", dirName, file->d_name, exception.reason);
-					}
-				} else {
-					imageDir = NULL;
-				}
-			} while (!image);
-
-			scaled_image = ScaleImage (image, text_x, text_y, &exception);
-			NormalizeImage(scaled_image);
-			ContrastImage(scaled_image, 0);
-			NegateImage(scaled_image, 0);
-
-			if (!next_pic)
-				next_pic = (unsigned char *)malloc (text_x * text_y);
-
-			ExportImagePixels (scaled_image, 0, 0, text_x, text_y, "I", CharPixel, next_pic, &exception);
-
-			DestroyImage (image);
-			DestroyImage (scaled_image);
-		}
-
 #ifdef HAVE_PTHREADS
-		void *imageLoadingThreadMain() {
-			pthread_mutex_lock(&next_pic_mutex);
-
-			do {
-				loadNextImageFromDisk();
-
-				pthread_cond_wait(&next_pic_cond, &next_pic_mutex);
-			} while (!exiting);
-
-			pthread_exit(NULL);
-		}
-
 		if (!imageLoadingThread) {
-			pthread_mutex_init(&next_pic_mutex, NULL);
-			pthread_cond_init(&next_pic_cond, NULL);
+			next_pic_mutex = (pthread_mutex_t *)malloc(sizeof(pthread_mutex_t));
+			pthread_mutex_init(next_pic_mutex, NULL);
+
+			pthread_mutex_lock(next_pic_mutex);
+
+			next_pic_cond = (pthread_cond_t *)malloc(sizeof(pthread_cond_t));
+			pthread_cond_init(next_pic_cond, NULL);
 
 			imageLoadingThread = (pthread_t *)malloc(sizeof(pthread_t));
 			pthread_create(imageLoadingThread, NULL, imageLoadingThreadMain, NULL);
 		} else {
-			pthread_mutex_lock(&next_pic_mutex);
+			pthread_mutex_lock(next_pic_mutex);
 			if (pic) {
 				unsigned char *tmp = next_pic;
 				next_pic = pic;
@@ -183,9 +210,9 @@ void loadNextImage ()
 				pic = next_pic;
 				next_pic = NULL;
 			}
-			pthread_mutex_unlock(&next_pic_mutex);
+			pthread_mutex_unlock(next_pic_mutex);
 
-			pthread_cond_signal(&next_pic_cond);
+			pthread_cond_signal(next_pic_cond);
 		}
 #else
 		next_pic = pic;
@@ -313,16 +340,21 @@ void draw_text ()
 			if (c > 255)
 				c = 255;
 
-			if (pic)
+			if (pic) {
 				c_pic = pic[p] * contrast - (255 - pic_fade);
-			if (c_pic < 0)
-				c_pic = 0;
-			c -= c_pic;
 
-			if (c < 0)
-				c = 0;
+				if (c_pic < 0)
+					c_pic = 0;
 
-			bump_pic[p] = (255 - c_pic) / 32.0f;
+				c -= c_pic;
+
+				if (c < 0)
+					c = 0;
+
+				bump_pic[p] = (255.0f - c_pic) / (256 / Z_Depth);
+			} else {
+				bump_pic[p] = Z_Depth;
+			}
 
 			if (c > 10)
 				if (text[p])
@@ -464,7 +496,7 @@ void hack_reshape (xstuff_t * XStuff)
 	glMatrixMode (GL_PROJECTION);
 	glLoadIdentity ();
 	//gluPerspective (45.0f, (GLfloat) XStuff->windowWidth / (GLfloat) XStuff->windowHeight, 0.1f, 150.0f);
-	glFrustum(-_text_x, _text_x, -_text_y, _text_y, 120, 128);
+	glFrustum(-_text_x, _text_x, -_text_y, _text_y, -Z_Off - Z_Depth, -Z_Off);
 
 	glMatrixMode (GL_MODELVIEW);
 }
@@ -475,7 +507,7 @@ void load_texture ()
 
 	LOAD_TEXTURE (font, cfont, cfont_compressedsize, cfont_size)
 
-		for (a = 0; a < 131072; a++) {
+	for (a = 0; a < 131072; a++) {
 		if ((a >> 9) & 2)
 			font[a] = font[a];
 		else
@@ -504,7 +536,6 @@ void ourBuildTextures ()
 	glBindTexture (GL_TEXTURE_2D, 1);
 
 	if ((gluerr = gluBuild2DMipmaps (GL_TEXTURE_2D, GL_RGBA8, 512, 256, GL_GREEN, GL_UNSIGNED_BYTE, (void *)font))) {
-
 		fprintf (stderr, "GLULib%s\n", gluErrorString (gluerr));
 		exit (-1);
 	}
@@ -512,20 +543,19 @@ void ourBuildTextures ()
 	glBindTexture (GL_TEXTURE_2D, 2);
 
 	if ((gluerr = gluBuild2DMipmaps (GL_TEXTURE_2D, GL_RGBA8, 512, 256, GL_LUMINANCE, GL_UNSIGNED_BYTE, (void *)font))) {
-
 		fprintf (stderr, "GLULib%s\n", gluErrorString (gluerr));
 		exit (-1);
 	}
 
 	FREE_TEXTURE (font)
 
-		glBindTexture (GL_TEXTURE_2D, 3);
+	glBindTexture (GL_TEXTURE_2D, 3);
 
 	if ((gluerr = gluBuild2DMipmaps (GL_TEXTURE_2D, GL_RGBA8, 4, 4, GL_LUMINANCE, GL_UNSIGNED_BYTE, (void *)flare))) {
-
 		fprintf (stderr, "GLULib%s\n", gluErrorString (gluerr));
 		exit (-1);
 	}
+
 	// Some pretty standard settings for wrapping and filtering.
 	glTexParameterf (GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
 	glTexParameterf (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
@@ -574,12 +604,12 @@ void hack_cleanup (xstuff_t * XStuff)
 #ifdef HAVE_PTHREADS
 	if (imageLoadingThread) {
 		// Try acquiring the lock first to make sure the thread is asleep and signallable. Otherwise the signal will miss the other thread and the pthread_join will hang.
-		pthread_mutex_lock(&next_pic_mutex);
+		pthread_mutex_lock(next_pic_mutex);
 
 		exiting = 1;
-		pthread_cond_signal(&next_pic_cond);
+		pthread_cond_signal(next_pic_cond);
 
-		pthread_mutex_unlock(&next_pic_mutex);
+		pthread_mutex_unlock(next_pic_mutex);
 
 		pthread_join(*imageLoadingThread, NULL);
 	}
@@ -587,23 +617,8 @@ void hack_cleanup (xstuff_t * XStuff)
 #endif
 }
 
-void hack_draw (xstuff_t * XStuff)
+void hack_draw (xstuff_t * XStuff, double currentTime, float frameTime)
 {
-	struct timeval now;
-	double dCurrentTime;
-	static double lastTime = -1;
-	double dDeltaTime;
-
-	gettimeofday (&now, NULL);
-
-	dCurrentTime = now.tv_sec + now.tv_usec * 0.000001;
-	if (lastTime == -1) {
-		dDeltaTime = 0;
-	} else {
-		dDeltaTime = dCurrentTime - lastTime;
-	}
-	lastTime = dCurrentTime;
-
 	glBindTexture (GL_TEXTURE_2D, 1);
 	glEnable (GL_BLEND);
 	glEnable (GL_TEXTURE_2D);
@@ -645,7 +660,7 @@ void hack_draw (xstuff_t * XStuff)
 	draw_flares ();
 	glEnd ();
 
-	make_change (dCurrentTime);
+	make_change (currentTime);
 
 	glLoadIdentity ();
 	glMatrixMode (GL_PROJECTION);
@@ -698,13 +713,26 @@ void hack_handle_opts (int argc, char **argv)
 
 #ifdef HAVE_IMAGEMAGICK
 		case 'i':
-			imageDir = opendir (optarg);
-			if (imageDir) {
-				dirName = (char *)malloc (strlen (optarg) + 1);
-				strcpy (dirName, optarg);
-			} else {
-				printf ("Could not access the directory specified with --images\n");
+			{
+				struct stat fileStat;
+
+				if (!stat(optarg, (struct stat *)&fileStat)) {
+					if (S_ISDIR(fileStat.st_mode)) {
+						imageDir = opendir (optarg);
+						if (imageDir) {
+							dirName = (char *)malloc (strlen (optarg) + 1);
+							strcpy (dirName, optarg);
+						} else {
+							fprintf (stderr, "--images: Could not open %s\n", optarg);
+						}
+					} else {
+						fprintf(stderr, "--images: %s is not a directory\n", optarg);
+					}
+				} else {
+					fprintf(stderr, "--images: Could not stat %s\n", optarg);
+				}
 			}
+
 			break;
 #endif
 		case 's':
@@ -727,11 +755,12 @@ void hack_handle_opts (int argc, char **argv)
 
 #ifdef HAVE_IMAGEMAGICK
 	// If contrast is unset, set it to 0.75 for external images, 1 for internal.
-	if (contrast == -1)
+	if (contrast == -1) {
 		if (dirName) 
 			contrast = 0.75;
 		else
 			contrast = 1;
+	}
 #else
 	// If contrast is unset, set it to 1.
 	if (contrast == -1)
