@@ -23,10 +23,8 @@
 char *hack_name = (char *)"euphoria";
 
 #include <stdio.h>
-#include <time.h>
 #include <GL/gl.h>
 #include <GL/glu.h>
-#include <sys/time.h>
 
 #include "driver.h"
 #include "euphoria_textures.h"
@@ -63,11 +61,9 @@ typedef struct {
 wisp *backwisps;
 wisp *wisps;
 unsigned int tex;
-unsigned char *feedbackmap;
 float aspectRatio;
 int viewport[4];
 float elapsedTime;
-int lastTime;
 
 /*
  * Parameters edited in the dialog box
@@ -110,24 +106,24 @@ void setDefaults (int which)
 	case PRESET_REGULAR:	/* Regular */
 		dWisps = 5;
 		dBgWisps = 0;
-		dDensity = 25;
+		dDensity = 35;
 		dVisibility = 35;
 		dSpeed = 15;
 		dFeedback = 0;
 		dFeedbackspeed = 1;
-		dFeedbacksize = 8;
+		dFeedbacksize = 10;
 		dWireframe = 0;
 		dTexture = 2;
 		break;
 	case PRESET_GRID:	/* Grid */
 		dWisps = 4;
 		dBgWisps = 1;
-		dDensity = 25;
+		dDensity = 30;
 		dVisibility = 70;
 		dSpeed = 15;
 		dFeedback = 0;
 		dFeedbackspeed = 1;
-		dFeedbacksize = 8;
+		dFeedbacksize = 10;
 		dWireframe = 1;
 		dTexture = 0;
 		break;
@@ -139,7 +135,7 @@ void setDefaults (int which)
 		dSpeed = 10;
 		dFeedback = 0;
 		dFeedbackspeed = 1;
-		dFeedbacksize = 8;
+		dFeedbacksize = 10;
 		dWireframe = 0;
 		dTexture = 0;
 		break;
@@ -151,19 +147,19 @@ void setDefaults (int which)
 		dSpeed = 30;
 		dFeedback = 40;
 		dFeedbackspeed = 5;
-		dFeedbacksize = 8;
+		dFeedbacksize = 10;
 		dWireframe = 1;
 		dTexture = 2;
 		break;
 	case PRESET_MTHEORY:	/* M-Theory */
 		dWisps = 3;
 		dBgWisps = 0;
-		dDensity = 25;
+		dDensity = 35;
 		dVisibility = 15;
 		dSpeed = 20;
 		dFeedback = 40;
 		dFeedbackspeed = 20;
-		dFeedbacksize = 8;
+		dFeedbacksize = 10;
 		dWireframe = 0;
 		dTexture = 0;
 		break;
@@ -175,7 +171,7 @@ void setDefaults (int which)
 		dSpeed = 50;
 		dFeedback = 0;
 		dFeedbackspeed = 1;
-		dFeedbacksize = 8;
+		dFeedbacksize = 10;
 		dWireframe = 0;
 		dTexture = 0;
 		break;
@@ -187,31 +183,31 @@ void setDefaults (int which)
 		dSpeed = 20;
 		dFeedback = 80;
 		dFeedbackspeed = 10;
-		dFeedbacksize = 8;
+		dFeedbacksize = 10;
 		dWireframe = 1;
 		dTexture = 3;
 		break;
 	case PRESET_ECHO:	/* Echo */
 		dWisps = 3;
 		dBgWisps = 0;
-		dDensity = 25;
+		dDensity = 35;
 		dVisibility = 30;
 		dSpeed = 20;
 		dFeedback = 85;
 		dFeedbackspeed = 30;
-		dFeedbacksize = 8;
+		dFeedbacksize = 10;
 		dWireframe = 0;
 		dTexture = 1;
 		break;
 	case PRESET_KALEIDOSCOPE:	/* Kaleidoscope */
 		dWisps = 3;
 		dBgWisps = 0;
-		dDensity = 25;
+		dDensity = 35;
 		dVisibility = 40;
 		dSpeed = 15;
 		dFeedback = 90;
 		dFeedbackspeed = 3;
-		dFeedbacksize = 8;
+		dFeedbacksize = 10;
 		dWireframe = 0;
 		dTexture = 0;
 		break;
@@ -244,19 +240,19 @@ void wisp_new (wisp * w)
 	 * initialize constants
 	 */
 	for (i = 0; i < NUMCONSTS; i++) {
-		w->c[i] = myRandf (2.0f) - 1.0f;
-		w->cr[i] = myRandf (PIx2);
-		w->cv[i] = myRandf (dSpeed * 0.03f) + (dSpeed * 0.001f);
+		w->c[i] = rsRandf (2.0f) - 1.0f;
+		w->cr[i] = rsRandf (PIx2);
+		w->cv[i] = rsRandf (dSpeed * 0.03f) + (dSpeed * 0.001f);
 	}
 
 	/*
 	 * pick color
 	 */
-	w->hsl[0] = myRandf (1.0f);
-	w->hsl[1] = 0.1f + myRandf (0.9f);
+	w->hsl[0] = rsRandf (1.0f);
+	w->hsl[1] = 0.1f + rsRandf (0.9f);
 	w->hsl[2] = 1.0f;
-	w->hueSpeed = myRandf (0.1f) - 0.05f;
-	w->saturationSpeed = myRandf (0.04f) + 0.001f;
+	w->hueSpeed = rsRandf (0.1f) - 0.05f;
+	w->saturationSpeed = rsRandf (0.04f) + 0.001f;
 }
 
 void delete_wisp (wisp * w)
@@ -446,27 +442,15 @@ void wisp_drawAsBackground (wisp * w)
 	glPopMatrix ();
 }
 
-void hack_draw (xstuff_t * XStuff)
+void hack_draw (xstuff_t * XStuff, double currentTime, float frameTime)
 {
 	int i;
-	int thisTime;
-	struct timeval now;
 #ifdef BENCHMARK
 	static int a = 1;
 #endif
 
-	/*
-	 * update time
-	 */
-	gettimeofday (&now, NULL);
-	thisTime = (now.tv_sec & 65535) * 1000 + now.tv_usec / 1000;
-
-	if (lastTime == -1)
-		lastTime = thisTime;
-
-	if (thisTime >= lastTime)
-		elapsedTime = (thisTime - lastTime) * 0.001f;
-	lastTime = thisTime;
+	if (frameTime > 0)
+		elapsedTime = frameTime;
 
 #ifdef BENCHMARK
 	elapsedTime = 0.001;
@@ -544,8 +528,7 @@ void hack_draw (xstuff_t * XStuff)
 		glReadBuffer (GL_BACK);
 		glPixelStorei (GL_UNPACK_ROW_LENGTH, feedbacktexsize);
 		glBindTexture (GL_TEXTURE_2D, feedbacktex);
-		glReadPixels (0, 0, feedbacktexsize, feedbacktexsize, GL_RGB, GL_UNSIGNED_BYTE, feedbackmap);
-		glTexSubImage2D (GL_TEXTURE_2D, 0, 0, 0, feedbacktexsize, feedbacktexsize, GL_RGB, GL_UNSIGNED_BYTE, feedbackmap);
+		glCopyTexSubImage2D (GL_TEXTURE_2D, 0, 0, 0, 0, 0, feedbacktexsize, feedbacktexsize);
 
 		/*
 		 * create regular drawing area
@@ -631,8 +614,6 @@ void hack_init (xstuff_t * XStuff)
 
 	rsCPUDetect();
 
-	lastTime = -1;
-
 	hack_reshape (XStuff);
 
 	if (dTexture) {
@@ -640,7 +621,7 @@ void hack_init (xstuff_t * XStuff)
 		unsigned char *l_tex = NULL;
 
 		if (whichtex == 4)	/* random texture */
-			whichtex = myRandi (3) + 1;
+			whichtex = rsRandi (3) + 1;
 		glEnable (GL_TEXTURE_2D);
 		glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
 		/*
@@ -678,25 +659,24 @@ void hack_init (xstuff_t * XStuff)
 		 * feedback texture setup
 		 */
 		glEnable (GL_TEXTURE_2D);
-		feedbackmap = (unsigned char *)malloc (feedbacktexsize * feedbacktexsize * 3);
 		glGenTextures (1, &feedbacktex);
 		glBindTexture (GL_TEXTURE_2D, feedbacktex);
 		glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 		glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
 		glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
 		glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
-		glTexImage2D (GL_TEXTURE_2D, 0, 3, feedbacktexsize, feedbacktexsize, 0, GL_RGB, GL_UNSIGNED_BYTE, feedbackmap);
+		glTexImage2D (GL_TEXTURE_2D, 0, 3, feedbacktexsize, feedbacktexsize, 0, GL_RGB, GL_UNSIGNED_BYTE, NULL);
 
 		/*
 		 * feedback velocity variable setup
 		 */
-		fv[0] = (float)dFeedbackspeed * (myRandf (0.025f) + 0.025f);
-		fv[1] = (float)dFeedbackspeed * (myRandf (0.05f) + 0.05f);
-		fv[2] = (float)dFeedbackspeed * (myRandf (0.05f) + 0.05f);
-		fv[3] = (float)dFeedbackspeed * (myRandf (0.1f) + 0.1f);
-		lv[0] = (float)dFeedbackspeed * (myRandf (0.0025f) + 0.0025f);
-		lv[1] = (float)dFeedbackspeed * (myRandf (0.0025f) + 0.0025f);
-		lv[2] = (float)dFeedbackspeed * (myRandf (0.0025f) + 0.0025f);
+		fv[0] = (float)dFeedbackspeed * (rsRandf (0.025f) + 0.025f);
+		fv[1] = (float)dFeedbackspeed * (rsRandf (0.05f) + 0.05f);
+		fv[2] = (float)dFeedbackspeed * (rsRandf (0.05f) + 0.05f);
+		fv[3] = (float)dFeedbackspeed * (rsRandf (0.1f) + 0.1f);
+		lv[0] = (float)dFeedbackspeed * (rsRandf (0.0025f) + 0.0025f);
+		lv[1] = (float)dFeedbackspeed * (rsRandf (0.0025f) + 0.0025f);
+		lv[2] = (float)dFeedbackspeed * (rsRandf (0.0025f) + 0.0025f);
 	}
 
 	/*
@@ -853,6 +833,6 @@ void hack_handle_opts (int argc, char **argv)
 	}
 
 	if (!change_flag) {
-		setDefaults (myRandi (9) + 1);
+		setDefaults (rsRandi (9) + 1);
 	}
 }
