@@ -23,10 +23,8 @@
 char *hack_name = (char *)"lattice";
 
 #include <stdio.h>
-#include <time.h>
 #include <GL/gl.h>
 #include <GL/glu.h>
-#include <sys/time.h>
 
 #include "driver.h"
 #include "lattice_textures.h"
@@ -45,8 +43,8 @@ char *hack_name = (char *)"lattice";
 /*
  * Where in the registry to store user defined variables 
  */
-#define NUMOBJECTS 10
-#define LATSIZE 10
+#define NUMOBJECTS 20
+#define LATSIZE 12
 #define TEXSIZE 256
 
 /*
@@ -82,7 +80,10 @@ typedef struct {
 
 camera ATTR_ALIGN(16) theCamera;
 
-int ATTR_ALIGN(8) lattice[LATSIZE][LATSIZE][LATSIZE];
+unsigned int ATTR_ALIGN(8) lattice[LATSIZE][LATSIZE][LATSIZE];
+unsigned int list_base;
+unsigned int texture_id[2];
+
 float bPnt[10][6];	/* Border points and direction
 			 * vectors where camera can cross
 			 * from cube to cube */
@@ -92,7 +93,6 @@ int lastBorder;
 int segments;
 
 float elapsedTime;
-unsigned int lastTime;
 
 /*
  * Parameters edited in the dialog box 
@@ -105,7 +105,6 @@ int dDrawdepth;
 int dFov;
 int dPathrand;
 int dSpeed;
-int dPriority;
 int dTexture;
 int dSmooth;
 int dFog;
@@ -259,95 +258,97 @@ void initTextures ()
 {
 	unsigned char *l_tex;
 
+	glGenTextures (2, texture_id);
+
 	switch (dTexture) {
 	case 1:
 		LOAD_TEXTURE (l_tex, indtex1, indtex1_compressedsize, indtex1_size)
-			glBindTexture (GL_TEXTURE_2D, 1);
+		glBindTexture (GL_TEXTURE_2D, texture_id[0]);
 		glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
 		glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 		glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
 		gluBuild2DMipmaps (GL_TEXTURE_2D, 3, TEXSIZE, TEXSIZE, GL_RGB, GL_UNSIGNED_BYTE, l_tex);
 		FREE_TEXTURE (l_tex)
 
-			LOAD_TEXTURE (l_tex, indtex2, indtex2_compressedsize, indtex2_size)
-			glBindTexture (GL_TEXTURE_2D, 2);
+		LOAD_TEXTURE (l_tex, indtex2, indtex2_compressedsize, indtex2_size)
+		glBindTexture (GL_TEXTURE_2D, texture_id[1]);
 		glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
 		glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 		glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
 		gluBuild2DMipmaps (GL_TEXTURE_2D, 3, TEXSIZE, TEXSIZE, GL_RGB, GL_UNSIGNED_BYTE, l_tex);
 		FREE_TEXTURE (l_tex)
 
-			break;
+		break;
 
 	case 2:
 		LOAD_TEXTURE (l_tex, crystex, crystex_compressedsize, crystex_size)
-			glBindTexture (GL_TEXTURE_2D, 1);
+		glBindTexture (GL_TEXTURE_2D, texture_id[0]);
 		glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
 		glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 		glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
 		gluBuild2DMipmaps (GL_TEXTURE_2D, 3, TEXSIZE, TEXSIZE, GL_RGB, GL_UNSIGNED_BYTE, l_tex);
 		FREE_TEXTURE (l_tex)
 
-			break;
+		break;
 
 	case 3:
 		LOAD_TEXTURE (l_tex, chrometex, chrometex_compressedsize, chrometex_size)
-			glBindTexture (GL_TEXTURE_2D, 1);
+		glBindTexture (GL_TEXTURE_2D, texture_id[0]);
 		glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
 		glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 		glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
 		gluBuild2DMipmaps (GL_TEXTURE_2D, 3, TEXSIZE, TEXSIZE, GL_RGB, GL_UNSIGNED_BYTE, l_tex);
 		FREE_TEXTURE (l_tex)
 
-			break;
+		break;
 
 	case 4:
 		LOAD_TEXTURE (l_tex, brasstex, brasstex_compressedsize, brasstex_size)
-			glBindTexture (GL_TEXTURE_2D, 1);
+		glBindTexture (GL_TEXTURE_2D, texture_id[0]);
 		glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
 		glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 		glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
 		gluBuild2DMipmaps (GL_TEXTURE_2D, 3, TEXSIZE, TEXSIZE, GL_RGB, GL_UNSIGNED_BYTE, l_tex);
 		FREE_TEXTURE (l_tex)
 
-			break;
+		break;
 
 	case 5:
 		LOAD_TEXTURE (l_tex, shinytex, shinytex_compressedsize, shinytex_size)
-			glBindTexture (GL_TEXTURE_2D, 1);
+		glBindTexture (GL_TEXTURE_2D, texture_id[0]);
 		glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
 		glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 		glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
 		gluBuild2DMipmaps (GL_TEXTURE_2D, 4, TEXSIZE, TEXSIZE, GL_RGBA, GL_UNSIGNED_BYTE, l_tex);
 		FREE_TEXTURE (l_tex)
 
-			break;
+		break;
 
 	case 6:
 		LOAD_TEXTURE (l_tex, ghostlytex, ghostlytex_compressedsize, ghostlytex_size)
-			glBindTexture (GL_TEXTURE_2D, 1);
+		glBindTexture (GL_TEXTURE_2D, texture_id[0]);
 		glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
 		glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 		glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
 		gluBuild2DMipmaps (GL_TEXTURE_2D, GL_ALPHA, TEXSIZE, TEXSIZE, GL_ALPHA, GL_UNSIGNED_BYTE, l_tex);
 		FREE_TEXTURE (l_tex)
 
-			break;
+		break;
 
 	case 7:
 		LOAD_TEXTURE (l_tex, circuittex, circuittex_compressedsize, circuittex_size)
-			glBindTexture (GL_TEXTURE_2D, 1);
+		glBindTexture (GL_TEXTURE_2D, texture_id[0]);
 		glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
 		glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 		glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
 		gluBuild2DMipmaps (GL_TEXTURE_2D, GL_ALPHA, TEXSIZE, TEXSIZE, GL_ALPHA, GL_UNSIGNED_BYTE, l_tex);
 		FREE_TEXTURE (l_tex)
 
-			break;
+		break;
 
 	case 8:
 		LOAD_TEXTURE (l_tex, doughtex, doughtex_compressedsize, doughtex_size)
-			glBindTexture (GL_TEXTURE_2D, 1);
+		glBindTexture (GL_TEXTURE_2D, texture_id[0]);
 		glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
 		glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 		glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
@@ -448,6 +449,13 @@ void makeTorus (int smooth, int longitude, int latitude, float centerradius, flo
 	}
 }
 
+void setMaterialAttribs(){
+	if(dTexture == 0 || dTexture >= 5)
+		glColor3f(rsRandf(1.0f), rsRandf(1.0f), rsRandf(1.0f));
+	if(dTexture == 1)
+		glBindTexture(GL_TEXTURE_2D, texture_id[rsRandi(2)]);
+}
+
 /*
  * Build the lattice display lists 
  */
@@ -456,27 +464,22 @@ void makeLatticeObjects ()
 	int i, d = 0;
 	float thick = (float)dThick * 0.001f;
 
-	if (dTexture > 0 && dTexture < 8)
-		glColor3f (1.0f, 1.0f, 1.0f);
+	list_base = glGenLists(NUMOBJECTS);
 
 	for (i = 1; i <= NUMOBJECTS; i++) {
-		glNewList (i, GL_COMPILE);
+		glNewList (list_base + i, GL_COMPILE);
 
 		if (dTexture >= 2)
-			glBindTexture (GL_TEXTURE_2D, 1);
+			glBindTexture (GL_TEXTURE_2D, texture_id[0]);
 
 		if (d < dDensity) {
 			glPushMatrix ();
 
-			if (dTexture == 0 || dTexture >= 5)
-				glColor3f (myRandf (1.0f), myRandf (1.0f), myRandf (1.0f));
-
-			if (dTexture == 1)
-				glBindTexture (GL_TEXTURE_2D, myRandi (2) + 1);
+			setMaterialAttribs();
 
 			glTranslatef (-0.25f, -0.25f, -0.25f);
 
-			if (myRandi (2))
+			if (rsRandi (2))
 				glRotatef (180.0f, 1, 0, 0);
 
 			makeTorus (dSmooth, dLongitude, dLatitude, 0.36f - thick, thick);
@@ -489,15 +492,11 @@ void makeLatticeObjects ()
 		if (d < dDensity) {
 			glPushMatrix ();
 
-			if (dTexture == 0 || dTexture >= 5)
-				glColor3f (myRandf (1.0f), myRandf (1.0f), myRandf (1.0f));
-
-			if (dTexture == 1)
-				glBindTexture (GL_TEXTURE_2D, myRandi (2) + 1);
+			setMaterialAttribs();
 
 			glTranslatef (0.25f, -0.25f, -0.25f);
 
-			if (myRandi (2))
+			if (rsRandi (2))
 				glRotatef (90.0f, 1, 0, 0);
 			else
 				glRotatef (-90.0f, 1, 0, 0);
@@ -512,15 +511,11 @@ void makeLatticeObjects ()
 		if (d < dDensity) {
 			glPushMatrix ();
 
-			if (dTexture == 0 || dTexture >= 5)
-				glColor3f (myRandf (1.0f), myRandf (1.0f), myRandf (1.0f));
-
-			if (dTexture == 1)
-				glBindTexture (GL_TEXTURE_2D, myRandi (2) + 1);
+			setMaterialAttribs();
 
 			glTranslatef (0.25f, -0.25f, 0.25f);
 
-			if (myRandi (2))
+			if (rsRandi (2))
 				glRotatef (90.0f, 0, 1, 0);
 			else
 				glRotatef (-90.0f, 0, 1, 0);
@@ -535,15 +530,11 @@ void makeLatticeObjects ()
 		if (d < dDensity) {
 			glPushMatrix ();
 
-			if (dTexture == 0 || dTexture >= 5)
-				glColor3f (myRandf (1.0f), myRandf (1.0f), myRandf (1.0f));
-
-			if (dTexture == 1)
-				glBindTexture (GL_TEXTURE_2D, myRandi (2) + 1);
+			setMaterialAttribs();
 
 			glTranslatef (0.25f, 0.25f, 0.25f);
 
-			if (myRandi (2))
+			if (rsRandi (2))
 				glRotatef (180.0f, 1, 0, 0);
 
 			makeTorus (dSmooth, dLongitude, dLatitude, 0.36f - thick, thick);
@@ -556,15 +547,11 @@ void makeLatticeObjects ()
 		if (d < dDensity) {
 			glPushMatrix ();
 
-			if (dTexture == 0 || dTexture >= 5)
-				glColor3f (myRandf (1.0f), myRandf (1.0f), myRandf (1.0f));
-
-			if (dTexture == 1)
-				glBindTexture (GL_TEXTURE_2D, myRandi (2) + 1);
+			setMaterialAttribs();
 
 			glTranslatef (-0.25f, 0.25f, 0.25f);
 
-			if (myRandi (2))
+			if (rsRandi (2))
 				glRotatef (90.0f, 1, 0, 0);
 			else
 				glRotatef (-90.0f, 1, 0, 0);
@@ -578,15 +565,11 @@ void makeLatticeObjects ()
 		if (d < dDensity) {
 			glPushMatrix ();
 
-			if (dTexture == 0 || dTexture >= 5)
-				glColor3f (myRandf (1.0f), myRandf (1.0f), myRandf (1.0f));
-
-			if (dTexture == 1)
-				glBindTexture (GL_TEXTURE_2D, myRandi (2) + 1);
+			setMaterialAttribs();
 
 			glTranslatef (-0.25f, 0.25f, -0.25f);
 
-			if (myRandi (2))
+			if (rsRandi (2))
 				glRotatef (90.0f, 0, 1, 0);
 			else
 				glRotatef (-90.0f, 0, 1, 0);
@@ -647,11 +630,11 @@ void reconfigure ()
 			globalxyz[2]--;
 	}
 
-	if (!myRandi (11 - dPathrand)) {	/* Change directions */
+	if (!rsRandi (11 - dPathrand)) {	/* Change directions */
 		if (!positive)
 			lastBorder += 10;
 
-		newBorder = transitions[lastBorder][myRandi (6)];
+		newBorder = transitions[lastBorder][rsRandi (6)];
 		positive = 0;
 		if (newBorder < 10)
 			positive = 1;
@@ -702,11 +685,11 @@ void reconfigure ()
 		for (j = 0; j < 3; j++) {
 			path[1][j] += globalxyz[j];
 			if ((newBorder < 6) && (j != 1))
-				path[1][j] += myRandf (0.15f) - 0.075f;
+				path[1][j] += rsRandf (0.15f) - 0.075f;
 		}
 
 		if (newBorder >= 6)
-			path[1][0] += myRandf (0.1f) - 0.05f;
+			path[1][0] += rsRandf (0.1f) - 0.05f;
 
 		segments = 1;
 	}
@@ -721,8 +704,6 @@ void draw ()
 	static float ATTR_ALIGN(16) oldDir[4] = { 0.0f, 0.0f, 0.0f, 0.0f };
 	static float ATTR_ALIGN(16) oldAngvel[4] = { 0.0f, 0.0f, 0.0f, 0.0f };
 	float angle, distance;
-	float rotationInertia = 0.007f * (float)dSpeed;
-	float maxSpin = 0.0025f * (float)dSpeed;
 	float ATTR_ALIGN(16) rotMat[16];
 	float ATTR_ALIGN(16) newQuat[4] = { 0.0f, 0.0f, 0.0f, 1.0f };
 	static float ATTR_ALIGN(16) quat[4] = { 0.0f, 0.0f, 0.0f, 1.0f };
@@ -732,7 +713,7 @@ void draw ()
 	static float where = 0.0f;	/* Position on path */
 	static float rollVel = 0.0f, rollAcc = 0.0f;
 	int drawDepth = dDrawdepth + 2;
-	static float rollChange = 0;	/* myRandf (10.0f) + 2.0f; */
+	static float rollChange = 0;	/* rsRandf (10.0f) + 2.0f; */
 
 	where += (float)dSpeed * 0.05f * elapsedTime;
 	if (where >= 1.0f) {
@@ -760,21 +741,14 @@ void draw ()
 	rsVec_cross ((float *)&angvel, dir, oldDir); /* Desired axis of rotation */
 
 	/* Protect against mild "overflow" */
-	angle = rsVec_dot (oldDir, dir);
-	if (angle >= 1.0f) {
-		angle = 0.0f;
-	} else {
-		angle = acos (rsVec_dot (oldDir, dir));	/* Desired turn angle */
-	}
-
-	if (angle > maxSpin) /* Cap the spin */
-		angle = maxSpin;
-	if (angle < -maxSpin)
-		angle = -maxSpin;
+	float dot = MAX(MIN(rsVec_dot (oldDir, dir), -1.0), 1.0);
+	float maxSpin = 0.25f * (float)dSpeed * elapsedTime;
+	angle = MAX(MIN(acos(dot), -maxSpin), maxSpin);
 
 	rsVec_scale ((float *)&angvel, angle); /* Desired angular velocity */
 	rsVec_subtract (angvel, oldAngvel, (float *)&tempVec); /* Change in angular velocity */
 	distance = rsVec_length (tempVec); /* Don't let angular velocity change too much */
+	float rotationInertia = 0.007f * (float)dSpeed * elapsedTime;
 	if (distance > rotationInertia * elapsedTime) {
 		rsVec_scale ((float *)&tempVec, ((rotationInertia * elapsedTime) / distance));
 		rsVec_add (oldAngvel, tempVec, (float *)&angvel);
@@ -787,8 +761,8 @@ void draw ()
 
 	if (flymodeChange <= 0.0f) {	/* transition from one fly mode to 
 					 * the other? */
-		flymode = myRandi (4);
-		flymodeChange = myRandf ((float)(150 - dSpeed)) + 5.0f;
+		flymode = rsRandi (4);
+		flymodeChange = rsRandf ((float)(150 - dSpeed)) + 5.0f;
 	}
 
 	rsVec_copy (angvel, (float *)&tempVec);	/* Recompute desired rotation */
@@ -803,8 +777,8 @@ void draw ()
 	/* Roll */
 	rollChange -= elapsedTime;
 	if (rollChange <= 0.0f) {
-		rollAcc = myRandf (0.02f * (float)dSpeed) - (0.01f * (float)dSpeed);
-		rollChange = myRandf (10.0f) + 2.0f;
+		rollAcc = rsRandf (0.02f * (float)dSpeed) - (0.01f * (float)dSpeed);
+		rollChange = rsRandf (10.0f) + 2.0f;
 	}
 
 	rollVel += rollAcc * elapsedTime;
@@ -833,9 +807,21 @@ void draw ()
 	/*
 	 * Apply transformations 
 	 */
+	glMatrixMode (GL_MODELVIEW);
 	glLoadMatrixf (rotMat);
 	glTranslatef (-xyz[0], -xyz[1], -xyz[2]);
 
+	// Just in case display lists contain no colors
+	glColor3f(1.0f, 1.0f, 1.0f);
+
+	// Environment mapping for crystal, chrome, brass, shiny, and ghostly
+	if(dTexture == 2 || dTexture == 3 || dTexture == 4  || dTexture == 5 || dTexture == 6){
+		glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
+		glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
+		glEnable(GL_TEXTURE_GEN_S);
+		glEnable(GL_TEXTURE_GEN_T);
+	}
+
 	/*
 	 * Render everything 
 	 */
@@ -895,34 +881,27 @@ void draw ()
 		}
 	}
 
+	glDisable(GL_TEXTURE_GEN_S);
+	glDisable(GL_TEXTURE_GEN_T);
+
 	glFlush ();
 }
 
-void hack_draw (xstuff_t * XStuff)
+void hack_draw (xstuff_t * XStuff, double currentTime, float frameTime)
 {
-	unsigned int thisTime;
 	static float times[10] = { 0.03f, 0.03f, 0.03f, 0.03f, 0.03f,
 		0.03f, 0.03f, 0.03f, 0.03f, 0.03f
 	};
 	static int timeindex = 0;
-	struct timeval now;
 #ifdef BENCHMARK
 	static int a = 1;
 #endif
 
-	/*
-	 * update time 
-	 */
-	gettimeofday (&now, NULL);
-	thisTime = (now.tv_sec & 65535) * 1000 + now.tv_usec / 1000;
-
-	if (thisTime >= lastTime)
-		times[timeindex] = (thisTime - lastTime) * 0.001f;
+	if (frameTime > 0)
+		times[timeindex] = frameTime;
 	else
 		times[timeindex] = elapsedTime;
 
-	lastTime = thisTime;
-
 #ifdef BENCHMARK
 	elapsedTime = 0.027;
 #else
@@ -969,20 +948,13 @@ void hack_reshape (xstuff_t * XStuff)
 void hack_init (xstuff_t * XStuff)
 {
 	int i, j, k;
-	struct timeval now;
 
 	rsCPUDetect();
 
-	/*
-	 * update time 
-	 */
-	gettimeofday (&now, NULL);
-	lastTime = (now.tv_sec & 65535) * 1000 + now.tv_usec / 1000;
-
 	hack_reshape (XStuff);
 
 	if (dTexture == 9)	/* Choose random texture */
-		dTexture = myRandi (9);
+		dTexture = rsRandi (9);
 
 	if (dTexture != 2 && dTexture != 6)	/* No z-buffering for crystal or ghostly */
 		glEnable (GL_DEPTH_TEST);
@@ -1038,16 +1010,6 @@ void hack_init (xstuff_t * XStuff)
 	if (dTexture) {
 		glEnable (GL_TEXTURE_2D);
 		initTextures ();
-		/*
-		 * Environment mapping for crystal, chrome, brass, shiny, and
-		 * ghostly 
-		 */
-		if (dTexture == 2 || dTexture == 3 || dTexture == 4 || dTexture == 5 || dTexture == 6) {
-			glTexGeni (GL_S, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
-			glTexGeni (GL_T, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
-			glEnable (GL_TEXTURE_GEN_S);
-			glEnable (GL_TEXTURE_GEN_T);
-		}
 	}
 
 	/*
@@ -1057,7 +1019,7 @@ void hack_init (xstuff_t * XStuff)
 	for (i = 0; i < LATSIZE; i++) {
 		for (j = 0; j < LATSIZE; j++) {
 			for (k = 0; k < LATSIZE; k++) {
-				lattice[i][j][k] = myRandi (NUMOBJECTS) + 1;
+				lattice[i][j][k] = list_base + rsRandi (NUMOBJECTS);
 			}
 		}
 	}
@@ -1142,7 +1104,7 @@ void hack_init (xstuff_t * XStuff)
 	path[0][4] = 0.0f;
 	path[0][5] = 0.0f;
 
-	j = myRandi (12);
+	j = rsRandi (12);
 	k = j % 6;
 	for (i = 0; i < 6; i++)
 		path[1][i] = bPnt[k][i];
@@ -1308,6 +1270,6 @@ void hack_handle_opts (int argc, char **argv)
 	}
 
 	if (!change_flag) {
-		setDefaults (myRandi (6) + 1);
+		setDefaults (rsRandi (6) + 1);
 	}
 }
